"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var aws_sdk_1 = require("aws-sdk");
var uuid = __importStar(require("uuid"));
var task_consumer_1 = require("./task-consumer");
var client_1 = require("./client");
var errors_1 = require("./errors");
var DEFAULT_QUEUE_NAME = 'default';
/**
 * Handles configuring queues, registering operations, and enqueueing/dequeueing tasks
 * for processing
 */
var AsyncTasksClient = /** @class */ (function () {
    function AsyncTasksClient(config) {
        var _a;
        this.queues = __assign({}, config.queues, (_a = {}, _a[DEFAULT_QUEUE_NAME] = config.defaultQueue, _a));
        this.sqsClient = config.sqsClient || new aws_sdk_1.SQS();
        this.routes = {};
    }
    Object.defineProperty(AsyncTasksClient.prototype, "registeredOperations", {
        get: function () {
            return Object.keys(this.routes);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Register an operation type with the asyncTask client
     *
     * Operations must have a name, a validation function, and a handler function and may
     * specify a queue on which jobs should be submitted by default
     *
     * @throws {QueueNotRegistered} the queueId specified is not registered
     */
    AsyncTasksClient.prototype.registerOperation = function (input) {
        var route = __assign({ queue: input.queue || DEFAULT_QUEUE_NAME }, input);
        if (typeof input.operationName !== 'string') {
            throw new TypeError('No operationName provided');
        }
        if (typeof input.validate !== 'function') {
            throw new TypeError('No validate function provided');
        }
        if (typeof input.handle !== 'function') {
            throw new TypeError('No handle function provided');
        }
        if (!this.queues[route.queue]) {
            throw new errors_1.QueueNotRegistered(route.queue);
        }
        this.routes[input.operationName] = route;
    };
    /**
     * Enqueues a task for a specified operation
     *
     * * Tasks must specify an operationName and a payload that can be serialized to JSON.
     * * The payload will be validated against the validation function provided when the
     *   operation was registered
     *
     * @throws {OperationNotRegistered} the operationName has not been configured
     * @throws {InvalidPayloadError} the provided payload did not pass validation
     * @throws {QueueNotRegistered} the specified queue is not configured
     * @returns the SQS MessageId and a unique taskId generated on our side
     */
    AsyncTasksClient.prototype.submitTask = function (input) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, queueName, task, queue, message, sqsResponse;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this._routeToTask(input)];
                    case 1:
                        _a = _b.sent(), queueName = _a.queueName, task = _a.task;
                        queue = this.queues[queueName];
                        message = {
                            QueueUrl: queue.queueUrl,
                            MessageBody: JSON.stringify(task),
                            DelaySeconds: input.delaySeconds
                        };
                        return [4 /*yield*/, this.sqsClient.sendMessage(message).promise()];
                    case 2:
                        sqsResponse = _b.sent();
                        return [2 /*return*/, {
                                messageId: sqsResponse.MessageId || null,
                                taskId: task.taskId
                            }];
                }
            });
        });
    };
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    AsyncTasksClient.prototype.submitAllTasks = function (input) {
        return __awaiter(this, void 0, void 0, function () {
            var routableTasks, messagesByQueue, responsesByTaskId, failedByTaskId, queueNames, sendPromises, results;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Promise.all(input.map(function (i) { return _this._routeToTask(i); }))
                        // Group tasks as messages assigned to their target queue
                    ];
                    case 1:
                        routableTasks = _a.sent();
                        messagesByQueue = {};
                        routableTasks.forEach(function (routableTask, i) {
                            var queueName = routableTask.queueName, task = routableTask.task;
                            if (!messagesByQueue[queueName]) {
                                messagesByQueue[queueName] = [];
                            }
                            messagesByQueue[queueName].push({
                                Id: task.taskId,
                                MessageBody: JSON.stringify(task),
                                DelaySeconds: input[i].delaySeconds
                            });
                        });
                        responsesByTaskId = {};
                        failedByTaskId = {};
                        queueNames = Object.keys(messagesByQueue);
                        sendPromises = queueNames.map(function (queueName) { return __awaiter(_this, void 0, void 0, function () {
                            var queueUrl, results;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        queueUrl = this.queues[queueName].queueUrl;
                                        return [4 /*yield*/, this.sqsClient
                                                .sendMessageBatch({
                                                QueueUrl: queueUrl,
                                                Entries: messagesByQueue[queueName]
                                            })
                                                .promise()];
                                    case 1:
                                        results = _a.sent();
                                        results.Successful.forEach(function (result) {
                                            responsesByTaskId[result.Id] = result;
                                        });
                                        results.Failed.forEach(function (result) {
                                            var error = {
                                                message: result.Message,
                                                code: result.Code,
                                                error: result
                                            };
                                            failedByTaskId[result.Id] = error;
                                        });
                                        return [2 /*return*/];
                                }
                            });
                        }); });
                        return [4 /*yield*/, Promise.all(sendPromises)
                            // Use the order of the routableTasks to generate an ordered array of responses
                        ];
                    case 2:
                        _a.sent();
                        results = routableTasks.map(function (routable) {
                            var taskId = routable.task.taskId;
                            if (failedByTaskId[taskId]) {
                                return { taskId: taskId, status: client_1.BatchSubmitTaskStatus.FAILED, error: failedByTaskId[taskId] };
                            }
                            return { taskId: taskId, status: client_1.BatchSubmitTaskStatus.SUCCESSFUL };
                        });
                        return [2 /*return*/, { results: results }];
                }
            });
        });
    };
    AsyncTasksClient.prototype.generateConsumers = function (input) {
        var _this = this;
        var queueNames = Object.keys(this.queues);
        var consumers = {};
        queueNames.forEach(function (queueName) {
            consumers[queueName] = task_consumer_1.createTaskConsumer({
                routes: _this.routes,
                contextProvider: input.contextProvider,
                consumerOptions: __assign({ sqs: _this.sqsClient }, input.consumerOpts, { queueUrl: _this.queues[queueName].queueUrl })
            });
        });
        return consumers;
    };
    AsyncTasksClient.prototype._routeToTask = function (input) {
        return __awaiter(this, void 0, void 0, function () {
            var operationName, payload, routeConfig, error_1, queueName, taskId, task;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        operationName = input.operationName, payload = input.payload;
                        routeConfig = this.routes[operationName];
                        if (!routeConfig) {
                            throw new errors_1.OperationNotRegistered(operationName);
                        }
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, routeConfig.validate(payload)];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        throw new errors_1.InvalidPayloadError(operationName, payload, error_1);
                    case 4:
                        queueName = routeConfig.queue || DEFAULT_QUEUE_NAME;
                        if (!this.queues[queueName]) {
                            throw new errors_1.QueueNotRegistered(queueName);
                        }
                        taskId = uuid.v4();
                        task = {
                            taskId: taskId,
                            operationName: operationName,
                            payload: payload
                        };
                        return [2 /*return*/, {
                                queueName: queueName,
                                task: task
                            }];
                }
            });
        });
    };
    return AsyncTasksClient;
}());
exports.AsyncTasksClient = AsyncTasksClient;
